// PasteTerm.cpp
// v1.01
// this code come from the TermHire add-on I made, "slightly" modified

#include <Directory.h>
#include <Window.h>
#include <StringView.h>
#include <File.h>
#include <Alert.h>
#include <Message.h>
#include <Debug.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <app/Clipboard.h>

// use this define if you want little windows to explain you what's going on
//#define VERBOSE 1

#define PASTE_LENGTH 60000
#define PATH_LENGTH 2048

#ifdef VERBOSE
BWindow *win;
BStringView *status;
#endif


#pragma export on
void process_refs(record_ref dir_ref, BMessage* msg, void*);
#pragma export reset
void put_in_clipboard(char *clip_string);
long add_name(char *paste, char * path);

static void error(const char *message)
{
	BAlert *alert = new BAlert("PasteTerm Error", message, "OK");
	alert->Go();
}

void put_in_clipboard(char *clip_string)
{
	be_clipboard->Lock();
	be_clipboard->Clear();
	be_clipboard->AddData(B_ASCII_TYPE,clip_string,strlen(clip_string));
	be_clipboard->Commit(); // this is necessary for our string to be saved in the clipboard
	be_clipboard->Unlock();
}

long add_name(char *paste, char *path)
{
	char path2[PATH_LENGTH+1];
	
	// I always add "" to avoid problems with special shell characters.
	// I know I could test if there are special chars in the strings, 
	// and only add "" if necessary, but I simply don't wanna test it

	// close your eyes, this is ugly
	// this is a bash-like completion : it adds a space char at the end of the string
	path2[0]='\"'; path2[1]='\0';
	strcat(path2,path);
	strcat(path2,"\" ");
	// you can open your eyes again
	if((strlen(paste)+strlen(path2))<PASTE_LENGTH)
	{
		strcat(paste,path2);
		return FALSE; // paste_full=FALSE
	}
	else
	{
		error("Sorry, I cannot handle THAT lot of references, try again with more "
			"little selections, or contact me for a more powerfull version : "
			"brua@pascal.u-strasbg.fr");
		return TRUE; //paste_full=TRUE
	}
}

void process_refs(record_ref dir_ref, BMessage* msg, void*)
{
	long count; // number of references stored in the BMessage
	ulong type;
	long numOK=0; // number of names analyzed correctly
	char path[PATH_LENGTH+1]; // used to get path strings
	char *paste; // paste string, filled during refs treatment
	bool paste_full=FALSE;

// this creates an informative window about the completion of the add-on
#ifdef VERBOSE
	BRect area(0,0, 250,15);
	screen_info info;

	get_screen_info(&info);
	area.OffsetBy(info.frame.Width()/2-125, info.frame.Height()/2-10);
	win = new BWindow(area, "PasteTerm Progress", B_TITLED_WINDOW,
		B_NOT_ZOOMABLE | B_NOT_RESIZABLE | B_NOT_CLOSABLE);
	status = new BStringView(win->Bounds(), "status", "PasteTerm");
	win->AddChild(status);
	win->Show();
#endif

	// init the paste string, before we begin adding names to it
	paste=(char *)malloc((PASTE_LENGTH+1)*sizeof(char));
	paste[0]='\0';

	// first, we consider the case when the add-on has been selected
	// from the file menu of the browser window or by right-clicking
	// in the background of the window
	// i.e : in this case, there is no refs on the BMessage
	
	if (!msg->GetInfo("refs", &type, &count))
	{
		BDirectory directory(dir_ref);
		directory.GetPath(path,PATH_LENGTH);
		if(!(paste_full=add_name(paste,path))) numOK++;
		put_in_clipboard(paste);
	}
	else
	{
		// if there are datas on the BMessage, but not from the ref type, exit...
		if (type != B_REF_TYPE) 
		{
			error("Cannot handle selection!");
			return;
		}
		else
		{
			for (int ix=0; ix<count; ix++) 
			{
				record_ref ref = msg->FindRef("refs", ix);
				// if the ref is a folder, add it to the selection
				if((does_ref_conform(ref,"Folder"))&&!paste_full)
				{
					BDirectory directory(ref);

					directory.GetPath(path,2048);
					if(!(paste_full=add_name(paste,path))) numOK++;
				}
				// if the ref is a file, add it to the selection
				else if((does_ref_conform(ref,"File"))&&!paste_full)
				{
					BFile file(ref);
		
					file.GetPath(path,PATH_LENGTH);
					if(!(paste_full=add_name(paste,path))) numOK++;
				}
			}
			if(paste_full)
				put_in_clipboard("PasteTerm : string too long"); // I clear the clipboard
			else
				put_in_clipboard(paste);
		}
	}
	
#ifdef VERBOSE
	sprintf(path, "%d out of %d names were stored in the clipboard", numOK, count);
	win->Lock();
	status->SetText(path);
	win->Unlock();
#endif
exit:
#ifdef VERBOSE
	snooze(100000);
	win->Quit();
#endif
	;
}

